@@ -40,6 +40,7 @@
 #include <common/na-pivot.h>
 #include <common/na-iio-provider.h>
 #include <common/na-ipivot-consumer.h>
+#include <common/na-iprefs.h>
 
 #include "nact-application.h"
 #include "nact-iactions-list.h"
@@ -54,20 +55,18 @@
 /* private class data
  */
 struct NactMainWindowClassPrivate {
+	void *empty;						/* so that gcc -pedantic is happy */
 };
 
 /* private instance data
  */
 struct NactMainWindowPrivate {
 	gboolean         dispose_has_run;
-	GtkStatusbar    *status_bar;
-	guint            status_context;
 	gint             initial_count;
 	GSList          *actions;
 	NAAction        *edited_action;
 	NAActionProfile *edited_profile;
 	GSList          *deleted;
-	GTimeVal         last_saved;
 };
 
 /* the GConf key used to read/write size and position of auxiliary dialogs
@@ -85,6 +84,7 @@ static void             iconditions_tab_iface_init( NactIConditionsTabInterface
 static void             iadvanced_tab_iface_init( NactIAdvancedTabInterface *iface );
 static void             imenubar_iface_init( NactIMenubarInterface *iface );
 static void             ipivot_consumer_iface_init( NAIPivotConsumerInterface *iface );
+static void             iprefs_iface_init( NAIPrefsInterface *iface );
 static void             instance_init( GTypeInstance *instance, gpointer klass );
 static void             instance_dispose( GObject *application );
 static void             instance_finalize( GObject *application );
@@ -92,18 +92,17 @@ static void             instance_finalize( GObject *application );
 static gchar           *get_iprefs_window_id( NactWindow *window );
 static gchar           *get_toplevel_name( BaseWindow *window );
 static GSList          *get_actions( NactWindow *window );
-static GtkWidget       *get_status_bar( NactWindow *window );
+static NAObject        *get_selected_object( NactWindow *window );
 
 static void             on_initial_load_toplevel( BaseWindow *window );
 static void             on_runtime_init_toplevel( BaseWindow *window );
 static void             setup_dialog_title( NactWindow *window );
 
-static void             on_actions_list_selection_changed( GtkTreeSelection *selection, gpointer user_data );
+static void             on_actions_list_selection_changed( NactIActionsList *instance, GSList *selected_items );
 static gboolean         on_actions_list_double_click( GtkWidget *widget, GdkEventButton *event, gpointer data );
-static gboolean         on_actions_list_delete_key_pressed( GtkWidget *widget, GdkEventKey *event, gpointer data );
 static gboolean         on_actions_list_enter_key_pressed( GtkWidget *widget, GdkEventKey *event, gpointer data );
-static void             set_current_action( NactMainWindow *window );
-static void             set_current_profile( NactMainWindow *window, gboolean set_action );
+static void             set_current_action( NactMainWindow *window, GSList *selected_items );
+static void             set_current_profile( NactMainWindow *window, gboolean set_action, GSList *selected_items );
 static NAAction        *get_edited_action( NactWindow *window );
 static NAActionProfile *get_edited_profile( NactWindow *window );
 static void             on_modified_field( NactWindow *window );
@@ -116,27 +115,22 @@ static void             get_isfiledir( NactWindow *window, gboolean *isfile, gbo
 static gboolean         get_multiple( NactWindow *window );
 static GSList          *get_schemes( NactWindow *window );
 
-static void             add_action( NactWindow *window, NAAction *action );
+static void             insert_item( NactWindow *window, NAAction *action );
 static void             add_profile( NactWindow *window, NAActionProfile *profile );
 static void             remove_action( NactWindow *window, NAAction *action );
 static GSList          *get_deleted_actions( NactWindow *window );
 static void             free_deleted_actions( NactWindow *window );
 static void             push_removed_action( NactWindow *window, NAAction *action );
 
-/*static void     on_new_button_clicked( GtkButton *button, gpointer user_data );
-static void     on_edit_button_clicked( GtkButton *button, gpointer user_data );
-static void     on_duplicate_button_clicked( GtkButton *button, gpointer user_data );
-static void     on_delete_button_clicked( GtkButton *button, gpointer user_data );
-static gboolean on_dialog_response( GtkDialog *dialog, gint response_id, BaseWindow *window );*/
-
 static void             update_actions_list( NactWindow *window );
 static gboolean         on_delete_event( BaseWindow *window, GtkWindow *toplevel, GdkEvent *event );
 static gint             count_actions( NactWindow *window );
 static gint             count_modified_actions( NactWindow *window );
 static void             reload_actions( NactWindow *window );
 static GSList          *free_actions( GSList *actions );
-static void             on_save( NactWindow *window );
 static void             on_actions_changed( NAIPivotConsumer *instance, gpointer user_data );
+static void             on_display_order_changed( NAIPivotConsumer *instance, gpointer user_data );
+static void             sort_actions_list( NactMainWindow *window );
 
 GType
 nact_main_window_get_type( void )
@@ -154,7 +148,7 @@ static GType
 register_type( void )
 {
 	static const gchar *thisfn = "nact_main_window_register_type";
-	g_debug( "%s", thisfn );
+	GType type;
 
 	static GTypeInfo info = {
 		sizeof( NactMainWindowClass ),
@@ -167,70 +161,74 @@ register_type( void )
 		0,
 		( GInstanceInitFunc ) instance_init
 	};
-	GType type = g_type_register_static( NACT_WINDOW_TYPE, "NactMainWindow", &info, 0 );
 
-	/* implement IActionsList interface
-	 */
 	static const GInterfaceInfo iactions_list_iface_info = {
 		( GInterfaceInitFunc ) iactions_list_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NACT_IACTIONS_LIST_TYPE, &iactions_list_iface_info );
 
-	/* implement IActionTab interface
-	 */
 	static const GInterfaceInfo iaction_tab_iface_info = {
 		( GInterfaceInitFunc ) iaction_tab_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NACT_IACTION_TAB_TYPE, &iaction_tab_iface_info );
 
-	/* implement ICommandTab interface
-	 */
 	static const GInterfaceInfo icommand_tab_iface_info = {
 		( GInterfaceInitFunc ) icommand_tab_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NACT_ICOMMAND_TAB_TYPE, &icommand_tab_iface_info );
 
-	/* implement IConditionsTab interface
-	 */
 	static const GInterfaceInfo iconditions_tab_iface_info = {
 		( GInterfaceInitFunc ) iconditions_tab_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NACT_ICONDITIONS_TAB_TYPE, &iconditions_tab_iface_info );
 
-	/* implement IAdvancedTab interface
-	 */
 	static const GInterfaceInfo iadvanced_tab_iface_info = {
 		( GInterfaceInitFunc ) iadvanced_tab_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NACT_IADVANCED_TAB_TYPE, &iadvanced_tab_iface_info );
 
-	/* implement IMenubar interface
-	 */
 	static const GInterfaceInfo imenubar_iface_info = {
 		( GInterfaceInitFunc ) imenubar_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NACT_IMENUBAR_TYPE, &imenubar_iface_info );
 
-	/* implement IPivotConsumer interface
-	 */
-	static const GInterfaceInfo pivot_consumer_iface_info = {
+	static const GInterfaceInfo ipivot_consumer_iface_info = {
 		( GInterfaceInitFunc ) ipivot_consumer_iface_init,
 		NULL,
 		NULL
 	};
-	g_type_add_interface_static( type, NA_IPIVOT_CONSUMER_TYPE, &pivot_consumer_iface_info );
+
+	static const GInterfaceInfo iprefs_iface_info = {
+		( GInterfaceInitFunc ) iprefs_iface_init,
+		NULL,
+		NULL
+	};
+
+	g_debug( "%s", thisfn );
+
+	type = g_type_register_static( NACT_WINDOW_TYPE, "NactMainWindow", &info, 0 );
+
+	g_type_add_interface_static( type, NACT_IACTIONS_LIST_TYPE, &iactions_list_iface_info );
+
+	g_type_add_interface_static( type, NACT_IACTION_TAB_TYPE, &iaction_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_ICOMMAND_TAB_TYPE, &icommand_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_ICONDITIONS_TAB_TYPE, &iconditions_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_IADVANCED_TAB_TYPE, &iadvanced_tab_iface_info );
+
+	g_type_add_interface_static( type, NACT_IMENUBAR_TYPE, &imenubar_iface_info );
+
+	g_type_add_interface_static( type, NA_IPIVOT_CONSUMER_TYPE, &ipivot_consumer_iface_info );
+
+	g_type_add_interface_static( type, NA_IPREFS_TYPE, &iprefs_iface_info );
 
 	return( type );
 }
@@ -239,23 +237,27 @@ static void
 class_init( NactMainWindowClass *klass )
 {
 	static const gchar *thisfn = "nact_main_window_class_init";
-	g_debug( "%s: klass=%p", thisfn, klass );
+	GObjectClass *object_class;
+	BaseWindowClass *base_class;
+	NactWindowClass *nact_class;
+
+	g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
 	st_parent_class = g_type_class_peek_parent( klass );
 
-	GObjectClass *object_class = G_OBJECT_CLASS( klass );
+	object_class = G_OBJECT_CLASS( klass );
 	object_class->dispose = instance_dispose;
 	object_class->finalize = instance_finalize;
 
 	klass->private = g_new0( NactMainWindowClassPrivate, 1 );
 
-	BaseWindowClass *base_class = BASE_WINDOW_CLASS( klass );
+	base_class = BASE_WINDOW_CLASS( klass );
 	base_class->get_toplevel_name = get_toplevel_name;
 	base_class->initial_load_toplevel = on_initial_load_toplevel;
 	base_class->runtime_init_toplevel = on_runtime_init_toplevel;
 	base_class->delete_event = on_delete_event;
 
-	NactWindowClass *nact_class = NACT_WINDOW_CLASS( klass );
+	nact_class = NACT_WINDOW_CLASS( klass );
 	nact_class->get_iprefs_window_id = get_iprefs_window_id;
 }
 
@@ -263,12 +265,13 @@ static void
 iactions_list_iface_init( NactIActionsListInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_iactions_list_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
 	iface->get_actions = get_actions;
 	iface->on_selection_changed = on_actions_list_selection_changed;
 	iface->on_double_click = on_actions_list_double_click;
-	iface->on_delete_key_pressed = on_actions_list_delete_key_pressed;
+	iface->on_delete_key_pressed = NULL;
 	iface->on_enter_key_pressed = on_actions_list_enter_key_pressed;
 	iface->is_modified_action = is_modified_action;
 	iface->is_valid_action = is_valid_action;
@@ -280,10 +283,9 @@ static void
 iaction_tab_iface_init( NactIActionTabInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_iaction_tab_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
 
-	iface->get_status_bar = get_status_bar;
-	iface->get_selected = nact_iactions_list_get_selected_object;
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+
 	iface->get_edited_action = get_edited_action;
 	iface->field_modified = on_modified_field;
 }
@@ -292,9 +294,9 @@ static void
 icommand_tab_iface_init( NactICommandTabInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_icommand_tab_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
 
-	iface->get_status_bar = get_status_bar;
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+
 	iface->get_edited_profile = get_edited_profile;
 	iface->field_modified = on_modified_field;
 	iface->get_isfiledir = get_isfiledir;
@@ -306,7 +308,8 @@ static void
 iconditions_tab_iface_init( NactIConditionsTabInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_iconditions_tab_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
 	iface->get_edited_profile = get_edited_profile;
 	iface->field_modified = on_modified_field;
@@ -316,7 +319,8 @@ static void
 iadvanced_tab_iface_init( NactIAdvancedTabInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_iadvanced_tab_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
 	iface->get_edited_profile = get_edited_profile;
 	iface->field_modified = on_modified_field;
@@ -326,43 +330,54 @@ static void
 imenubar_iface_init( NactIMenubarInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_imenubar_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
 
-	iface->add_action = add_action;
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+
+	iface->insert_item = insert_item;
 	iface->add_profile = add_profile;
 	iface->remove_action = remove_action;
 	iface->get_deleted_actions = get_deleted_actions;
 	iface->free_deleted_actions = free_deleted_actions;
 	iface->push_removed_action = push_removed_action;
 	iface->get_actions = get_actions;
-	iface->get_selected = nact_iactions_list_get_selected_object;
-	iface->get_status_bar = get_status_bar;
+	iface->get_selected = get_selected_object;
 	iface->setup_dialog_title = setup_dialog_title;
 	iface->update_actions_list = update_actions_list;
 	iface->select_actions_list = nact_iactions_list_set_selection;
 	iface->count_actions = count_actions;
 	iface->count_modified_actions = count_modified_actions;
 	iface->reload_actions = reload_actions;
-	iface->on_save = on_save;
 }
 
 static void
 ipivot_consumer_iface_init( NAIPivotConsumerInterface *iface )
 {
 	static const gchar *thisfn = "nact_main_window_ipivot_consumer_iface_init";
-	g_debug( "%s: iface=%p", thisfn, iface );
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
 	iface->on_actions_changed = on_actions_changed;
+	iface->on_display_order_changed = on_display_order_changed;
+}
+
+static void
+iprefs_iface_init( NAIPrefsInterface *iface )
+{
+	static const gchar *thisfn = "nact_main_window_iprefs_iface_init";
+
+	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 }
 
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
 	static const gchar *thisfn = "nact_main_window_instance_init";
-	g_debug( "%s: instance=%p, klass=%p", thisfn, instance, klass );
+	NactMainWindow *self;
+
+	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
 
 	g_assert( NACT_IS_MAIN_WINDOW( instance ));
-	NactMainWindow *self = NACT_MAIN_WINDOW( instance );
+	self = NACT_MAIN_WINDOW( instance );
 
 	self->private = g_new0( NactMainWindowPrivate, 1 );
 
@@ -373,17 +388,20 @@ static void
 instance_dispose( GObject *window )
 {
 	static const gchar *thisfn = "nact_main_window_instance_dispose";
-	g_debug( "%s: window=%p", thisfn, window );
+	NactMainWindow *self;
+	GtkWidget *pane;
+	gint pos;
 
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
 	g_assert( NACT_IS_MAIN_WINDOW( window ));
-	NactMainWindow *self = NACT_MAIN_WINDOW( window );
+	self = NACT_MAIN_WINDOW( window );
 
 	if( !self->private->dispose_has_run ){
 
 		self->private->dispose_has_run = TRUE;
 
-		GtkWidget *pane = base_window_get_widget( BASE_WINDOW( window ), "MainPaned" );
-		gint pos = gtk_paned_get_position( GTK_PANED( pane ));
+		pane = base_window_get_widget( BASE_WINDOW( window ), "MainPaned" );
+		pos = gtk_paned_get_position( GTK_PANED( pane ));
 		nact_iprefs_set_int( NACT_WINDOW( window ), "main-paned", pos );
 
 		self->private->actions = free_actions( self->private->actions );
@@ -403,13 +421,12 @@ static void
 instance_finalize( GObject *window )
 {
 	static const gchar *thisfn = "nact_main_window_instance_finalize";
-	g_debug( "%s: window=%p", thisfn, window );
+	NactMainWindow *self;
+
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
 
 	g_assert( NACT_IS_MAIN_WINDOW( window ));
-	NactMainWindow *self = ( NactMainWindow * ) window;
-
-	/*g_free( self->private->current_uuid );
-	g_free( self->private->current_label );*/
+	self = ( NactMainWindow * ) window;
 
 	g_free( self->private );
 
@@ -446,6 +463,7 @@ gboolean
 nact_main_window_action_exists( const NactMainWindow *window, const gchar *uuid )
 {
 	GSList *ia;
+
 	for( ia = window->private->actions ; ia ; ia = ia->next ){
 		NAAction *action = NA_ACTION( ia->data );
 		gchar *action_uuid = na_action_get_uuid( action );
@@ -455,9 +473,25 @@ nact_main_window_action_exists( const NactMainWindow *window, const gchar *uuid
 			return( TRUE );
 		}
 	}
+
 	return( FALSE );
 }
 
+/**
+ * Returns the status bar widget
+ */
+GtkStatusbar *
+nact_main_window_get_statusbar( const NactMainWindow *window )
+{
+	GtkWidget *statusbar;
+
+	g_assert( NACT_IS_MAIN_WINDOW( window ));
+
+	statusbar = base_window_get_widget( BASE_WINDOW( window ), "StatusBar" );
+
+	return( GTK_STATUSBAR( statusbar ));
+}
+
 static gchar *
 get_iprefs_window_id( NactWindow *window )
 {
@@ -477,11 +511,19 @@ get_actions( NactWindow *window )
 	return( NACT_MAIN_WINDOW( window )->private->actions );
 }
 
-static GtkWidget *
-get_status_bar( NactWindow *window )
+static NAObject *
+get_selected_object( NactWindow *window )
 {
-	g_assert( NACT_IS_MAIN_WINDOW( window ));
-	return( GTK_WIDGET( NACT_MAIN_WINDOW( window )->private->status_bar ));
+	NAObject *object;
+	GSList *items;
+
+	object = NULL;
+	items = nact_iactions_list_get_selected_items( NACT_IACTIONS_LIST( window ));
+	if( g_slist_length( items ) == 1 ){
+		object = NA_OBJECT( items->data );
+	}
+
+	return( object );
 }
 
 /*
@@ -494,26 +536,31 @@ static void
 on_initial_load_toplevel( BaseWindow *window )
 {
 	static const gchar *thisfn = "nact_main_window_on_initial_load_toplevel";
+	NactMainWindow *wnd;
+	gint pos;
+	GtkWidget *pane;
+	/*gboolean alpha_order;*/
 
 	/* call parent class at the very beginning */
 	if( BASE_WINDOW_CLASS( st_parent_class )->initial_load_toplevel ){
 		BASE_WINDOW_CLASS( st_parent_class )->initial_load_toplevel( window );
 	}
 
-	g_debug( "%s: window=%p", thisfn, window );
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
 	g_assert( NACT_IS_MAIN_WINDOW( window ));
-	NactMainWindow *wnd = NACT_MAIN_WINDOW( window );
-
-	wnd->private->status_bar = GTK_STATUSBAR( base_window_get_widget( window, "StatusBar" ));
-	wnd->private->status_context = gtk_statusbar_get_context_id( wnd->private->status_bar, "nact-main-window" );
+	wnd = NACT_MAIN_WINDOW( window );
 
 	nact_imenubar_init( wnd );
 
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
 	nact_iactions_list_initial_load( NACT_WINDOW( window ));
 	nact_iactions_list_set_edition_mode( NACT_WINDOW( window ), TRUE );
-	nact_iactions_list_set_multiple_selection( NACT_WINDOW( window ), FALSE );
 	nact_iactions_list_set_send_selection_changed_on_fill_list( NACT_WINDOW( window ), FALSE );
+	nact_iactions_list_set_multiple_selection( NACT_WINDOW( window ), TRUE );
+
+	/*alpha_order = na_iprefs_get_alphabetical_order( NA_IPREFS( window ));
+	nact_iactions_list_set_multiple_selection( NACT_WINDOW( window ), !alpha_order );
+	nact_iactions_list_set_dnd_mode( NACT_WINDOW( window ), !alpha_order );*/
 
 	g_assert( NACT_IS_IACTION_TAB( window ));
 	nact_iaction_tab_initial_load( NACT_WINDOW( window ));
@@ -527,9 +574,9 @@ on_initial_load_toplevel( BaseWindow *window )
 	g_assert( NACT_IS_IADVANCED_TAB( window ));
 	nact_iadvanced_tab_initial_load( NACT_WINDOW( window ));
 
-	gint pos = nact_iprefs_get_int( NACT_WINDOW( window ), "main-paned" );
+	pos = nact_iprefs_get_int( NACT_WINDOW( window ), "main-paned" );
 	if( pos ){
-		GtkWidget *pane = base_window_get_widget( window, "MainPaned" );
+		pane = base_window_get_widget( window, "MainPaned" );
 		gtk_paned_set_position( GTK_PANED( pane ), pos );
 	}
 }
@@ -538,6 +585,10 @@ static void
 on_runtime_init_toplevel( BaseWindow *window )
 {
 	static const gchar *thisfn = "nact_main_window_on_runtime_init_toplevel";
+	NactMainWindow *wnd;
+	NactApplication *application;
+	NAPivot *pivot;
+	GSList *ia;
 
 	/* call parent class at the very beginning */
 	if( BASE_WINDOW_CLASS( st_parent_class )->runtime_init_toplevel ){
@@ -545,10 +596,11 @@ on_runtime_init_toplevel( BaseWindow *window )
 	}
 
 	g_assert( NACT_IS_MAIN_WINDOW( window ));
-	NactMainWindow *wnd = NACT_MAIN_WINDOW( window );
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
+	wnd = NACT_MAIN_WINDOW( window );
 
-	NactApplication *application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( wnd )));
-	NAPivot *pivot = nact_application_get_pivot( application );
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( wnd )));
+	pivot = nact_application_get_pivot( application );
 	na_pivot_set_automatic_reload( pivot, FALSE );
 	wnd->private->actions = na_pivot_get_duplicate_actions( pivot );
 	wnd->private->initial_count = g_slist_length( wnd->private->actions );
@@ -556,17 +608,10 @@ on_runtime_init_toplevel( BaseWindow *window )
 	/* initialize the current edition status as a loaded action may be
 	 * invalid (without having been modified)
 	 */
-	GSList *ia;
 	for( ia = wnd->private->actions ; ia ; ia = ia->next ){
 		na_object_check_edited_status( NA_OBJECT( ia->data ));
 	}
 
-	g_get_current_time( &wnd->private->last_saved );
-
-	g_debug( "%s: window=%p", thisfn, window );
-	g_assert( NACT_IS_MAIN_WINDOW( window ));
-	/*NactMainWindow *wnd = NACT_MAIN_WINDOW( window );*/
-
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
 	nact_iactions_list_runtime_init( NACT_WINDOW( window ));
 
@@ -585,15 +630,18 @@ on_runtime_init_toplevel( BaseWindow *window )
 	/* forces a no-selection when the list is initially empty
 	 */
 	if( !wnd->private->initial_count ){
-		set_current_action( NACT_MAIN_WINDOW( window ));
+		set_current_action( NACT_MAIN_WINDOW( window ), NULL );
+	} else {
+		nact_iactions_list_select_first( NACT_WINDOW( window ));
 	}
 }
 
 static void
 setup_dialog_title( NactWindow *window )
 {
+	GtkWindow *toplevel;
 	BaseApplication *appli = BASE_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	gchar *title = base_application_get_name( appli );
+	gchar *title = base_application_get_application_name( appli );
 
 	if( NACT_MAIN_WINDOW( window )->private->edited_action ){
 		gchar *label = na_action_get_label( NACT_MAIN_WINDOW( window )->private->edited_action );
@@ -609,41 +657,42 @@ setup_dialog_title( NactWindow *window )
 		title = tmp;
 	}
 
-	GtkWindow *toplevel = base_window_get_toplevel_dialog( BASE_WINDOW( window ));
+	toplevel = base_window_get_toplevel_dialog( BASE_WINDOW( window ));
 	gtk_window_set_title( toplevel, title );
 
 	g_free( title );
 }
 
-/*
- * note that the IActionsList tree store may return an action or a profile
- */
 static void
-on_actions_list_selection_changed( GtkTreeSelection *selection, gpointer user_data )
+on_actions_list_selection_changed( NactIActionsList *instance, GSList *selected_items )
 {
 	static const gchar *thisfn = "nact_main_window_on_actions_list_selection_changed";
-	g_debug( "%s: selection=%p, user_data=%p", thisfn, selection, user_data );
+	NactMainWindow *window;
+	NAObject *object;
+	gint count;
 
-	g_assert( NACT_IS_MAIN_WINDOW( user_data ));
-	NactMainWindow *window = NACT_MAIN_WINDOW( user_data );
+	g_debug( "%s: instance=%p, selected_items=%p", thisfn, ( void * ) instance, ( void * ) selected_items );
 
-	NAObject *object = nact_iactions_list_get_selected_object( NACT_WINDOW( window ));
-	g_debug( "%s: object=%p", thisfn, object );
+	g_assert( NACT_IS_MAIN_WINDOW( instance ));
+	window = NACT_MAIN_WINDOW( instance );
 
-	if( object ){
+	count = g_slist_length( selected_items );
+
+	if( count == 1 ){
+		object = NA_OBJECT( selected_items->data );
 		if( NA_IS_ACTION( object )){
 			window->private->edited_action = NA_ACTION( object );
-			set_current_action( window );
+			set_current_action( window, selected_items );
 
 		} else {
 			g_assert( NA_IS_ACTION_PROFILE( object ));
 			window->private->edited_profile = NA_ACTION_PROFILE( object );
-			set_current_profile( window, TRUE );
+			set_current_profile( window, TRUE, selected_items );
 		}
 
 	} else {
 		window->private->edited_action = NULL;
-		set_current_action( window );
+		set_current_action( window, selected_items );
 	}
 }
 
@@ -658,17 +707,6 @@ on_actions_list_double_click( GtkWidget *widget, GdkEventButton *event, gpointer
 	return( TRUE );
 }
 
-static gboolean
-on_actions_list_delete_key_pressed( GtkWidget *widget, GdkEventKey *event, gpointer user_data )
-{
-	if( NACT_MAIN_WINDOW( user_data )->private->edited_action ){
-		nact_imenubar_on_delete_key_pressed( NACT_WINDOW( user_data ));
-		return( TRUE );
-	}
-
-	return( FALSE );
-}
-
 static gboolean
 on_actions_list_enter_key_pressed( GtkWidget *widget, GdkEventKey *event, gpointer user_data )
 {
@@ -683,11 +721,14 @@ on_actions_list_enter_key_pressed( GtkWidget *widget, GdkEventKey *event, gpoint
  * if there is only one profile, we also setup the profile
  */
 static void
-set_current_action( NactMainWindow *window )
+set_current_action( NactMainWindow *window, GSList *selected_items )
 {
-	g_debug( "set_current_action: current=%p", window->private->edited_action );
+	static const gchar *thisfn = "nact_main_window_set_current_action";
 
-	nact_iaction_tab_set_action( NACT_WINDOW( window ), window->private->edited_action );
+	g_debug( "%s: window=%p, current=%p, selected_items=%p",
+			thisfn, ( void * ) window, ( void * ) window->private->edited_action, ( void * ) selected_items );
+
+	nact_iaction_tab_set_action( NACT_WINDOW( window ), window->private->edited_action, selected_items );
 
 	window->private->edited_profile = NULL;
 
@@ -697,16 +738,21 @@ set_current_action( NactMainWindow *window )
 		}
 	}
 
-	set_current_profile( window, FALSE );
+	set_current_profile( window, FALSE, selected_items );
 }
 
 static void
-set_current_profile( NactMainWindow *window, gboolean set_action )
+set_current_profile( NactMainWindow *window, gboolean set_action, GSList *selected_items )
 {
+	static const gchar *thisfn = "nact_main_window_set_current_profile";
+
+	g_debug( "%s: window=%p, set_action=%s, selected_items=%p",
+			thisfn, ( void * ) window, set_action ? "True":"False", ( void * ) selected_items );
+
 	if( window->private->edited_profile && set_action ){
 		NAAction *action = NA_ACTION( na_action_profile_get_action( window->private->edited_profile ));
 		window->private->edited_action = action;
-		nact_iaction_tab_set_action( NACT_WINDOW( window ), window->private->edited_action );
+		nact_iaction_tab_set_action( NACT_WINDOW( window ), window->private->edited_action, selected_items );
 	}
 
 	nact_icommand_tab_set_profile( NACT_WINDOW( window ), window->private->edited_profile );
@@ -792,11 +838,44 @@ get_schemes( NactWindow *window )
 	return( nact_iadvanced_tab_get_schemes( window ));
 }
 
+/*
+ * insert an item (action or menu) in the list:
+ * - the last position if the list is sorted, and sort it
+ * - at the current position if the list is not sorted
+ *
+ * set the selection on the new item
+ */
 static void
-add_action( NactWindow *window, NAAction *action )
+insert_item( NactWindow *window, NAAction *item )
 {
 	NactMainWindow *wnd = NACT_MAIN_WINDOW( window );
-	wnd->private->actions = g_slist_prepend( wnd->private->actions, ( gpointer ) action );
+	gboolean alpha_order;
+	gchar *uuid;
+	NAAction *current;
+	gint index;
+
+	alpha_order = na_iprefs_get_alphabetical_order( NA_IPREFS( window ));
+	if( alpha_order ){
+		wnd->private->actions = g_slist_prepend( wnd->private->actions, ( gpointer ) item );
+		sort_actions_list( wnd );
+
+	} else {
+		current = get_edited_action( window );
+		if( current ){
+			index = g_slist_index( wnd->private->actions, current );
+			g_assert( index >= 0 );
+			wnd->private->actions = g_slist_insert( wnd->private->actions, item, index );
+		} else {
+			g_assert( g_slist_length( wnd->private->actions ) == 0 );
+			wnd->private->actions = g_slist_prepend( wnd->private->actions, ( gpointer ) item );
+		}
+	}
+
+	nact_iactions_list_fill( window, TRUE );
+
+	uuid = na_action_get_uuid( item );
+	nact_iactions_list_set_selection( window, NA_ACTION_TYPE, uuid, NULL );
+	g_free( uuid );
 }
 
 static void
@@ -837,162 +916,6 @@ push_removed_action( NactWindow *window, NAAction *action )
 	wnd->private->deleted = g_slist_append( wnd->private->deleted, ( gpointer ) action );
 }
 
-/*
- * creating a new action
- * pwi 2009-05-19
- * I don't want the profiles feature spread wide while I'm not convinced
- * that it is useful and actually used.
- * so the new action is silently created with a default profile name
- */
-/*static void
-on_new_button_clicked( GtkButton *button, gpointer user_data )
-{
-	static const gchar *thisfn = "nact_main_window_on_new_button_clicked";
-	g_debug( "%s: button=%p, user_data=%p", thisfn, button, user_data );
-
-	g_assert( NACT_IS_MAIN_WINDOW( user_data ));
-	NactWindow *wndmain = NACT_WINDOW( user_data );
-
-	nact_action_conditions_editor_run_editor( wndmain, NULL );
-
-	nact_iactions_list_set_focus( wndmain );
-}*/
-
-/*
- * editing an existing action
- * pwi 2009-05-19
- * I don't want the profiles feature spread wide while I'm not convinced
- * that it is useful and actually used.
- * so :
- * - if there is only one profile, the user will be directed to a dialog
- *   box which includes all needed fields, but without any profile notion
- * - if there are more than one profile, one can assume that the user has
- *   found a use to the profiles, and let him edit them
- */
-/*static void
-on_edit_button_clicked( GtkButton *button, gpointer user_data )
-{
-	static const gchar *thisfn = "nact_main_window_on_edit_button_clicked";
-	g_debug( "%s: button=%p, user_data=%p", thisfn, button, user_data );
-
-	g_assert( NACT_IS_MAIN_WINDOW( user_data ));
-	NactWindow *wndmain = NACT_WINDOW( user_data );
-
-	NAAction *action = NA_ACTION( nact_iactions_list_get_selected_action( wndmain ));
-	g_assert( action );
-	g_assert( NA_IS_ACTION( action ));
-
-	if( na_action_get_profiles_count( action ) > 1 ){
-		nact_action_profiles_editor_run_editor( wndmain, action );
-	} else {
-		nact_action_conditions_editor_run_editor( wndmain, action );
-	}
-
-	nact_iactions_list_set_focus( wndmain );
-}*/
-
-/*static void
-on_duplicate_button_clicked( GtkButton *button, gpointer user_data )
-{
-	static const gchar *thisfn = "nact_main_window_on_duplicate_button_clicked";
-	g_debug( "%s: button=%p, user_data=%p", thisfn, button, user_data );
-
-	g_assert( NACT_IS_MAIN_WINDOW( user_data ));
-	NactWindow *wndmain = NACT_WINDOW( user_data );
-
-	NAAction *action = NA_ACTION( nact_iactions_list_get_selected_action( wndmain ));
-	if( action ){
-
-		NAAction *duplicate = na_action_duplicate( action );
-		na_action_set_new_uuid( duplicate );
-		gchar *label = na_action_get_label( action );
-		gchar *label2 = g_strdup_printf( _( "Copy of %s"), label );
-		na_action_set_label( duplicate, label2 );
-		g_free( label2 );
-
-		gchar *msg = NULL;
-		NAPivot *pivot = NA_PIVOT( nact_window_get_pivot( wndmain ));
-		if( na_pivot_write_action( pivot, G_OBJECT( duplicate ), &msg ) != NA_IIO_PROVIDER_WRITE_OK ){
-
-			gchar *first = g_strdup_printf( _( "Unable to duplicate \"%s\" action." ), label );
-			base_window_error_dlg( BASE_WINDOW( wndmain ), GTK_MESSAGE_ERROR, first, msg );
-			g_free( first );
-			g_free( msg );
-
-		} else {
-			do_set_current_action( NACT_WINDOW( wndmain ), duplicate );
-		}
-
-		g_object_unref( duplicate );
-		g_free( label );
-
-	} else {
-		g_assert_not_reached();
-	}
-
-	nact_iactions_list_set_focus( wndmain );
-}*/
-
-/*static void
-on_delete_button_clicked( GtkButton *button, gpointer user_data )
-{
-	static const gchar *thisfn = "nact_main_window_on_delete_button_clicked";
-	g_debug( "%s: button=%p, user_data=%p", thisfn, button, user_data );
-
-	g_assert( NACT_IS_MAIN_WINDOW( user_data ));
-	NactWindow *wndmain = NACT_WINDOW( user_data );
-
-	NAAction *action = NA_ACTION( nact_iactions_list_get_selected_action( wndmain ));
-	if( action ){
-
-		gchar *label = na_action_get_label( action );
-		gchar *sure = g_strdup_printf( _( "Are you sure you want to delete \"%s\" action ?" ), label );
-		if( base_window_yesno_dlg( BASE_WINDOW( wndmain ), GTK_MESSAGE_WARNING, sure, NULL )){
-
-			gchar *msg = NULL;
-			NAPivot *pivot = NA_PIVOT( nact_window_get_pivot( wndmain ));
-			if( na_pivot_delete_action( pivot, G_OBJECT( action ), &msg ) != NA_IIO_PROVIDER_WRITE_OK ){
-
-				gchar *first = g_strdup_printf( _( "Unable to delete \"%s\" action." ), label );
-				base_window_error_dlg( BASE_WINDOW( wndmain ), GTK_MESSAGE_ERROR, first, msg );
-				g_free( first );
-				g_free( msg );
-			}
-		}
-		g_free( sure );
-		g_free( label );
-
-	} else {
-		g_assert_not_reached();
-	}
-
-	nact_iactions_list_set_focus( wndmain );
-}*/
-
-/*static gboolean
-on_dialog_response( GtkDialog *dialog, gint response_id, BaseWindow *window )
-{
-	static const gchar *thisfn = "nact_main_window_on_dialog_response";
-	g_debug( "%s: dialog=%p, response_id=%d, window=%p", thisfn, dialog, response_id, window );
-	g_assert( NACT_IS_MAIN_WINDOW( window ));
-*/
-	/*GtkWidget *paste_button = nact_get_glade_widget_from ("PasteProfileButton", GLADE_EDIT_DIALOG_WIDGET);*/
-
-	/*switch( response_id ){
-		case GTK_RESPONSE_NONE:
-		case GTK_RESPONSE_DELETE_EVENT:
-		case GTK_RESPONSE_CLOSE:*/
-			/* Free any profile in the clipboard */
-			/*nautilus_actions_config_action_profile_free (g_object_steal_data (G_OBJECT (paste_button), "profile"));*/
-
-			/*g_object_unref( window );
-			return( TRUE );
-			break;
-	}
-
-	return( FALSE );
-}*/
-
 static void
 update_actions_list( NactWindow *window )
 {
@@ -1003,9 +926,11 @@ static gboolean
 on_delete_event( BaseWindow *window, GtkWindow *toplevel, GdkEvent *event )
 {
 	static const gchar *thisfn = "nact_main_window_on_delete_event";
-	g_debug( "%s: window=%p, toplevel=%p, event=%p", thisfn, window, toplevel, event );
 
+	g_debug( "%s: window=%p, toplevel=%p, event=%p",
+			thisfn, ( void * ) window, ( void * ) toplevel, ( void * ) event );
 	g_assert( NACT_IS_MAIN_WINDOW( window ));
+
 	nact_imenubar_on_delete_event( NACT_WINDOW( window ));
 
 	return( TRUE );
@@ -1028,14 +953,14 @@ count_actions( NactWindow *window )
 static gint
 count_modified_actions( NactWindow *window )
 {
+	GSList *ia;
+	gint count = 0;
+
 	if( g_slist_length( NACT_MAIN_WINDOW( window )->private->actions ) == 0 &&
 		NACT_MAIN_WINDOW( window )->private->initial_count == 0 ){
 			return( 0 );
 	}
 
-	GSList *ia;
-	gint count = 0;
-
 	for( ia = NACT_MAIN_WINDOW( window )->private->deleted ; ia ; ia = ia->next ){
 		if( na_object_get_origin( NA_OBJECT( ia->data )) != NULL ){
 			count += 1;
@@ -1055,11 +980,14 @@ static void
 reload_actions( NactWindow *window )
 {
 	NactMainWindow *self = NACT_MAIN_WINDOW( window );
+	NactApplication *application;
+	NAPivot *pivot;
+
 	self->private->actions = free_actions( self->private->actions );
 	self->private->deleted = free_actions( self->private->deleted );
 
-	NactApplication *application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-	NAPivot *pivot = nact_application_get_pivot( application );
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+	pivot = nact_application_get_pivot( application );
 	na_pivot_reload_actions( pivot );
 	self->private->actions = na_pivot_get_duplicate_actions( pivot );
 	self->private->initial_count = g_slist_length( self->private->actions );
@@ -1069,8 +997,6 @@ reload_actions( NactWindow *window )
 	if( self->private->initial_count ){
 		nact_iactions_list_select_first( window );
 	}
-	/*self->private->edited_action = NULL;
-	set_current_action( NACT_MAIN_WINDOW( window ));*/
 }
 
 static GSList *
@@ -1084,29 +1010,6 @@ free_actions( GSList *actions )
 	return( NULL );
 }
 
-/*
- * in initial_load_toplevel(), we have forbidden the automatic reload
- * of the list of actions in NAPivot, as we take care of updating it of
- * the modifications entered in the UI
- *
- * this doesn't prevent NAPivot to advertise us when it detects some
- * modifications in an I/O provider ; so that we are able to ask the
- * user for a reload
- *
- * but we don't want be advertized when this is our own save which
- * triggers the NAPivot advertising - so we forbids such advertisings
- * during one seconde after each save
- *
- * note that last_saved is initialized in initial_load_toplevel()
- * so we will not be advertised if NAPivot detects a modification
- * in the seconde after this initialization - just ignore this case
- */
-static void
-on_save( NactWindow *window )
-{
-	g_get_current_time( &NACT_MAIN_WINDOW( window )->private->last_saved );
-}
-
 /*
  * called by NAPivot because this window implements the IIOConsumer
  * interface, i.e. it wish to be advertised when the list of actions
@@ -1120,23 +1023,20 @@ static void
 on_actions_changed( NAIPivotConsumer *instance, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_main_window_on_actions_changed";
-	g_debug( "%s: instance=%p, user_data=%p", thisfn, instance, user_data );
+	NactMainWindow *self;
+	NactApplication *application;
+	NAPivot *pivot;
+	gchar *first, *second;
+	gboolean ok;
 
+	g_debug( "%s: instance=%p, user_data=%p", thisfn, ( void * ) instance, ( void * ) user_data );
 	g_assert( NACT_IS_MAIN_WINDOW( instance ));
-	NactMainWindow *self = NACT_MAIN_WINDOW( instance );
+	self = NACT_MAIN_WINDOW( instance );
 
-	GTimeVal now;
-	g_get_current_time( &now );
-	glong ecart = 1000000 * ( now.tv_sec - self->private->last_saved.tv_sec );
-	ecart += now.tv_usec - self->private->last_saved.tv_usec;
-	if( ecart < 1000000 ){
-		return;
-	}
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
+	pivot = nact_application_get_pivot( application );
 
-	NactApplication *application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
-	NAPivot *pivot = nact_application_get_pivot( application );
-
-	gchar *first = g_strdup(_( "One or more actions have been modified in the filesystem.\n"
+	first = g_strdup(_( "One or more actions have been modified in the filesystem.\n"
 								"You could keep to work with your current list of actions, "
 								"or you may want to reload a fresh one." ));
 
@@ -1148,9 +1048,9 @@ on_actions_changed( NAIPivotConsumer *instance, gpointer user_data )
 		first = tmp;
 	}
 
-	gchar *second = g_strdup( _( "Do you want to reload a fresh list of actions ?" ));
+	second = g_strdup( _( "Do you want to reload a fresh list of actions ?" ));
 
-	gboolean ok = base_window_yesno_dlg( BASE_WINDOW( instance ), GTK_MESSAGE_QUESTION, first, second );
+	ok = base_window_yesno_dlg( BASE_WINDOW( instance ), GTK_MESSAGE_QUESTION, first, second );
 
 	g_free( second );
 	g_free( first );
@@ -1166,3 +1066,35 @@ on_actions_changed( NAIPivotConsumer *instance, gpointer user_data )
 		nact_iactions_list_fill( NACT_WINDOW( instance ), TRUE );
 	}
 }
+
+/*
+ * called by NAPivot via NAIPivotConsumer whenever the
+ * "sort in alphabetical order" preference is modified.
+ */
+static void
+on_display_order_changed( NAIPivotConsumer *instance, gpointer user_data )
+{
+	static const gchar *thisfn = "nact_main_window_on_display_order_changed";
+	/*NactMainWindow *self;*/
+	gboolean alpha_order;
+
+	g_debug( "%s: instance=%p, user_data=%p", thisfn, ( void * ) instance, ( void * ) user_data );
+	g_assert( NACT_IS_MAIN_WINDOW( instance ));
+	/*self = NACT_MAIN_WINDOW( instance );*/
+
+	alpha_order = na_iprefs_get_alphabetical_order( NA_IPREFS( instance ));
+
+	nact_iactions_list_set_multiple_selection( NACT_WINDOW( instance ), !alpha_order );
+	nact_iactions_list_set_dnd_mode( NACT_WINDOW( instance ), !alpha_order );
+}
+
+static void
+sort_actions_list( NactMainWindow *window )
+{
+	NactApplication *application;
+	NAPivot *pivot;
+
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+	pivot = nact_application_get_pivot( application );
+	window->private->actions = na_iio_provider_sort_actions( pivot, window->private->actions );
+}
